<?php
namespace Teknisa\FIN\Modules\ContaReceber\Services\Remessa\Banco;

use Financeiro\Entities\Emprbanc;
use Financeiro\Entities\Boletreceber;
use Geral\Entities\Banco;
use Geral\Entities\Endeclie;
use Geral\Entities\Licencauso;
use Geral\Entities\Contcorr;
use \Geral\Entities\Empresa;
use Geral\Util\Ambiente;
use Teknisa\Libs\Util\DateUtil;
use \Financeiro\Util\Util;
use \Geral\Entities\Cliente;
use Geral\Entities\Filial;
use Teknisa\FIN\Modules\ContaReceber\Services\Remessa\GeraCnabService;

class GeraCnabBrasilService extends GeraCnabService{

    const CODIGO_BANCO = '001';
    const LOTE_DO_SERVICO = '0001';
    const LOTE_DO_SERVICO_HEADER = '0000';
    const LOTE_DO_SERVICO_TRAILLER = '9999';
    const ID_ARQUIVO = '0';
    const VERSAO_ARQUIVO = '030';
    const VERSAO_ARQUIVO_2 = '020';
    const COBRANCA_SEM_PAPEL = 'CSP';
    const USO_VANS = '000';
    const CODIGO_ARQUIVO = '1';
    const TIPO_DE_REGISTRO = '1';
    const TIPO_DE_OPERACAO = 'R';
    const TIPO_DE_SERVICO = '01';
    const CODIGO_MOEDA = '09';
    const CODIGO_PARA_BAIXA = '2';
    const CODIGO_DESCONTO = '0';
    const TIPO_DE_REGISTRO_DETALHE = '3';
    const TIPO_DE_REGISTRO_TRAILLER_LOTE = '5';
    const TIPO_DE_REGISTRO_TRAILLER = '9';
    const CODIGO_SEGMENTO_P = 'P';
    const CODIGO_SEGMENTO_Q = 'Q';
    const CODIGO_SEGMENTO_R = 'R';
    const TIPO_DOCUMENTO = '2';
    const ESPECIE_TITULO = '02';
    const ID_TITULO_ACEITO = 'N';

    public function geraHeaderArquivo($caminho, $nmarquivo, $nrorg, $cdbanco, $cdempresa, $cdidrec, $nrseqremessarec, $nrcnab, $cdreducont, $carteira = NULL, $contreg = NULL, $idtipcnabrec = NULL, $cdfilial, $contador = NULL){
        
        if (!is_dir($caminho))
            mkdir($caminho, "0777", true);
        $nmarquivo = $caminho.$nmarquivo;
        $file = fopen($nmarquivo, "w+");

        // CAMPO: Código do Banco
        // POSIÇÃO: 001 a 003
        // PICTURE: 9(3)
        $arquivo = self::CODIGO_BANCO;

        // CAMPO: Lote de Serviço
        // POSIÇÃO: 004 a 007
        // PICTURE: 9(4)
        $arquivo .= self::LOTE_DO_SERVICO_HEADER;

        // CAMPO: Identificação do Arquivo
        // POSIÇÃO: 008 a 008
        // PICTURE: 9(1)
        $arquivo .= self::ID_ARQUIVO;

        // CAMPO: Uso exclusivo FEBRABAN
        // POSIÇÃO: 009 a 017
        // PICTURE: X(9)
        $arquivo = str_pad($arquivo, 17);

        // CAMPO: Tipo da inscrição da empresa
        // POSIÇÃO: 018 a 018
        // PICTURE: 9(1)
        /** @var $empresa Empresa */
        
        $empresa = Empresa::findOneByCdempresa($cdempresa);
        $filial = Filial::findOneByCdfilial($cdfilial);
        
        if($empresa){
            if($empresa->getIdtpijurempr() == 'F')
                $tipoInscricaoCedente = '1';
            else
                $tipoInscricaoCedente = '2';
        }
        
        $arquivo .= $tipoInscricaoCedente;

        // CAMPO: Nº de inscrição da empresa
        // POSIÇÃO: 019 a 032
        // PICTURE: 9(15)
        
        if($filial) {
            $arquivo .= $filial->getNrinsjurfili();
        } else {
            $arquivo .= $empresa->getNrinsjurempr();
        }
 
        // if($empresa){
        //     $arquivo .= $empresa->getNrinsjurempr();
        // }

        // CAMPO: Convênio            000035165 0014 51 019 bb
        // POSIÇÃO: 033 a 052
        // PICTURE: X(20)
        // 09 - Código do Convênio
        // 04 - Código do Produto      "0014"
        // 02 - Carteira               "N Carteira"
        // 03 - Variação da Carteira   "019"
        // 02 - Integração SAP         "Vazio"

        /** @var $emprbanc Emprbanc */
        $emprbanc = Emprbanc::findOneByCdempresaAndCdbanco($cdempresa, $cdbanco);
        $idenRec = $this->getIdContRec($cdreducont, $cdbanco, $cdempresa);
        //Variação da Carteira de Cobrança
        if (($emprbanc && !$emprbanc->getNrvariacartcob()) || !$emprbanc){
            $varCartCob = '019';
        }
        else{
            $varCartCob = $emprbanc->getNrvariacartcob();
        }
        if($idenRec){
            $arquivo .= str_pad(Util::campoCnabValido(trim($idenRec)), 9, '0', STR_PAD_LEFT) . '0014' . substr($carteira, -2) . $varCartCob . '  ';
        }
        else
           $arquivo = str_pad($arquivo, 52);

        // CAMPO: Agência mantenedora da conta
        // POSIÇÃO: 053 a 057
        // PICTURE: X(6)
        /** @var $contcorr Contcorr */
        $contcorr = Contcorr::findOneByCdreducont($cdreducont);
        if($contcorr && $contcorr->getCdagencia())
            $arquivo .= str_pad(substr($contcorr->getCdagencia(), 0, 4), 5, '0', STR_PAD_LEFT);
        else
            $arquivo = str_pad($arquivo, 57, '0');

        // CAMPO: Dígito verificador
        // POSIÇÃO: 058 a 058
        // PICTURE: X(1)
        if($contcorr)
            $arquivo .= substr($contcorr->getCdagencia(), -1);

        // CAMPO: Conta Corrente
        // POSIÇÃO: 059 a 70
        // PICTURE: X(13)
        if($contcorr && $contcorr->getCdcontcorr()){
            $tamanho = strlen(str_replace(' ', '', $contcorr->getCdcontcorr())) - 1;
            $arquivo .= str_pad(substr(str_replace(' ', '', $contcorr->getCdcontcorr()), 0, $tamanho), 12, '0', STR_PAD_LEFT);
        }
        else
            $arquivo = str_pad($arquivo, 70, '0');

        // CAMPO: Dígito verificador da conta
        // POSIÇÃO: 071 a 71
        // PICTURE: X(1)
        if($contcorr)
            $arquivo .= substr(str_replace(' ', '', $contcorr->getCdcontcorr()), -1);

        // CAMPO: Dígito Verificador da Agência/Conta
        // POSIÇÃO: 072 a 072
        // PICTURE: X(1)
        $arquivo = str_pad($arquivo, 72);

        // CAMPO: Nome da Empresa
        // POSIÇÃO: 073 a 0102
        // PICTURE: X(30)
        /** @var $empresa Empresa */
        $empresa = Empresa::findOneByCdempresa($cdempresa);
        $filial = Filial::findOneByCdfilial($cdfilial);

        if($filial){
            $nmrazsocemp = Util::campoCnabValido($filial->getNmrazsocfili());
            $nmrazsocemp = substr($nmrazsocemp, 0, 30);
            $arquivo .= str_pad($nmrazsocemp, 30);
        } else {
           $nmrazsocemp = Util::campoCnabValido($empresa->getNmrazsocempr());
           $nmrazsocemp = substr($nmrazsocemp, 0, 30);
           $arquivo .= str_pad($nmrazsocemp, 30); 
        }

        // if ($empresa) {
        //     $nmrazsocemp = Util::campoCnabValido($empresa->getNmrazsocempr());
        //     $nmrazsocemp = substr($nmrazsocemp, 0, 30);
        //     $arquivo .= str_pad($nmrazsocemp, 30);
        // }
        // else
        //     $arquivo = str_pad($arquivo, 102);

        // CAMPO: Nome do Banco
        // POSIÇÃO: 103 a 132
        // PICTURE: X(30)
        if($contcorr){
            /** @var $banco Banco */
            $banco = Banco::findOneByCdbanco($contcorr->getCdbanco());
            if($banco){
                $nmbanco = Util::campoCnabValido($banco->getNmbanco());
                $nmbanco = substr($nmbanco, 0, 30);
                $arquivo .= str_pad($nmbanco, 30);
            }
        }
        else
            $arquivo = str_pad($arquivo, 132);

        // CAMPO: Brancos
        // POSIÇÃO: 133 a 142
        // PICTURE: X(10)
        $arquivo = str_pad($arquivo, 142);

        // CAMPO: Código do Arquivo
        // POSIÇÃO: 143 a 143
        // PICTURE: X(1)
        $arquivo .= self::CODIGO_ARQUIVO; // REMESSA

        // CAMPO: Data de Geração do arquivo
        // POSIÇÃO: 144 a 151
        // PICTURE: 9(8)
        $now = DateUtil::getDataDeString(DateUtil::truncateString(DateUtil::getDataAtual()));
        $data = DateUtil::getDia($now) . DateUtil::getMes($now) . DateUtil::getAno($now);
        $arquivo .= Util::campoCnabValido($data);

        // CAMPO: Hora de Geração do arquivo
        // POSIÇÃO: 152 a 157
        // PICTURE: 9(6)
        $now = DateUtil::getDataAtual();
        $horaGeracao = $now->format('His');
        $arquivo .= $horaGeracao;


        // CAMPO: Número Seqüencial
        // POSIÇÃO: 158 a 163
        // PICTURE: 9(6)
        $arquivo .= substr($nrseqremessarec, -6);

        // CAMPO: Versão do Arquivo
        // POSIÇÃO: 164 a 166
        // PICTURE: X(3)
        $arquivo .= self::VERSAO_ARQUIVO;

        // CAMPO: Densidade de gravação do arquivo
        // POSIÇÃO: 167 a 171
        // PICTURE: X(5)
        $arquivo = str_pad($arquivo, 171, '0');

        // CAMPO: Reservado para o Banco
        // POSIÇÃO: 172 a 191
        // PICTURE: X(20)
        $arquivo = str_pad($arquivo, 191);

        // CAMPO: Reservado para a Empresa
        // POSIÇÃO: 192 a 211
        // PICTURE: X(20)
        $arquivo = str_pad($arquivo, 211);

        // CAMPO: Brancos
        // POSIÇÃO: 212 a 222
        // PICTURE: X(11)
        $arquivo = str_pad($arquivo, 222);

        // CAMPO: Conbranca s/ papel
        // POSIÇÃO: 212 a 225
        // PICTURE: X(3)
        $arquivo .= self::COBRANCA_SEM_PAPEL;

        // CAMPO: Uso exclusivo das Vans
        // POSIÇÃO: 226 a 228
        // PICTURE: X(3)
        $arquivo .= self::USO_VANS;

        // CAMPO: Brancos
        // POSIÇÃO: 229 a 240
        // PICTURE: X(12)
        $arquivo = str_pad($arquivo, 240) . "\r\n";

        //Grava a variavel arquivo no arquivo texto
        fprintf($file, "%s", $arquivo);
        fclose($file);
    }
    public function geraHeaderLote($caminho, $nmarquivo, $nrorg, $cdbanco, $cdempresa, $cdidrec, $nrseqremessarec, $nrcnabrec, $cdreducont, $carteira = NULL, $cdfilial){

        if (!is_dir($caminho))
            mkdir($caminho, "0777", true);
        $nmarquivo = $caminho.$nmarquivo;
        $file = fopen($nmarquivo, "a+");

        // CAMPO: Código do Banco
        // POSIÇÃO: 001 a 003
        // PICTURE: 9(3)
        $arquivo = self::CODIGO_BANCO;

        // CAMPO: Lote de Serviço
        // POSIÇÃO: 004 a 007
        // PICTURE: 9(4)
        $arquivo .= self::LOTE_DO_SERVICO;

        // CAMPO: Tipo de Registro
        // POSIÇÃO: 008 a 008
        // PICTURE: 9(1)
        $arquivo .= self::TIPO_DE_REGISTRO;

        // CAMPO: Tipo de Operação
        // POSIÇÃO: 009 a 009
        // PICTURE: X(1)
        $arquivo .= self::TIPO_DE_OPERACAO;

        // CAMPO: Tipo de Serviço
        // POSIÇÃO: 010 a 011
        // PICTURE: 9(2)
        $arquivo .= self::TIPO_DE_SERVICO;

        // CAMPO: Forma de Lançamento
        // POSIÇÃO: 012 a 013
        // PICTURE: 9(2)
        $arquivo = str_pad($arquivo, 13, '0');

        // CAMPO: Versão do Arquivo
        // POSIÇÃO: 014 a 016
        // PICTURE: X(3)
        $arquivo .= self::VERSAO_ARQUIVO_2;

        // CAMPO: Brancos
        // POSIÇÃO: 017 a 017
        // PICTURE: 9(1)
        $arquivo = str_pad($arquivo, 17);

        // CAMPO: Tipo da inscrição da empresa
        // POSIÇÃO: 018 a 018
        // PICTURE: 9(1)
        /** @var $empresa Empresa */
        $empresa = Empresa::findOneByCdempresa($cdempresa);
        $filial = Filial::findOneByCdfilial($cdfilial);
        if($empresa){
            if($empresa->getIdtpijurempr() == 'F')
                $tipoInscricaoCedente = '1';
            else
                $tipoInscricaoCedente = '2';
        }
        $arquivo .= $tipoInscricaoCedente;

        // CAMPO: Nº de inscrição da empresa
        // POSIÇÃO: 019 a 033
        // PICTURE: 9(15)
        
        if($filial){
            $arquivo .= str_pad($filial->getNrinsjurfili(), 15, '0', STR_PAD_LEFT);
        } else {
            $arquivo .= str_pad($empresa->getNrinsjurempr(), 15, '0', STR_PAD_LEFT);
        }
        
        // if($empresa){
        //     $arquivo .= str_pad($empresa->getNrinsjurempr(), 15, '0', STR_PAD_LEFT);
        // }

        // CAMPO: Convênio
        // POSIÇÃO: 034 a 053
        // PICTURE: X(20)
        // 09 - Código do Convênio
        // 04 - Código do Produto      "0014"
        // 02 - Carteira               "N Carteira"
        // 03 - Variação da Carteira   "019"
        // 02 - Integração SAP         "Vazio"

        /** @var $emprbanc Emprbanc */
        $emprbanc = Emprbanc::findOneByCdempresaAndCdbanco($cdempresa, $cdbanco);

        $idenRec = $this->getIdContRec($cdreducont, $cdbanco, $cdempresa);
        //Variação da Carteira de Cobrança
        if (($emprbanc && !$emprbanc->getNrvariacartcob()) || !$emprbanc)
            $varCartCob = '019';
        else
            $varCartCob = $emprbanc->getNrvariacartcob();
        if($idenRec){
            $arquivo .= str_pad(Util::campoCnabValido(trim($idenRec)), 9, '0', STR_PAD_LEFT) . '0014' . substr($carteira, -2) . $varCartCob . '  ';
        }
        else
            $arquivo = str_pad($arquivo, 53);

        // CAMPO: Agência mantenedora da conta
        // POSIÇÃO: 054 a 058
        // PICTURE: X(6)
        /** @var $contcorr Contcorr */
        $contcorr = Contcorr::findOneByCdreducont($cdreducont);
        if($contcorr && $contcorr->getCdagencia())
            $arquivo .= str_pad(substr($contcorr->getCdagencia(), 0, 4), 5, '0', STR_PAD_LEFT);
        else
            $arquivo = str_pad($arquivo, 58, '0');

        // CAMPO: Dígito verificador
        // POSIÇÃO: 059 a 059
        // PICTURE: X(1)
        if($contcorr)
            $arquivo .= substr($contcorr->getCdagencia(), -1);

        // CAMPO: Conta Corrente
        // POSIÇÃO: 060 a 71
        // PICTURE: X(12)
        if($contcorr && $contcorr->getCdcontcorr()){
            $tamanho = strlen(str_replace(' ', '', $contcorr->getCdcontcorr())) - 1;
            $arquivo .= str_pad(substr(str_replace(' ', '', $contcorr->getCdcontcorr()), 0, $tamanho), 12, '0', STR_PAD_LEFT);
        }
        else
            $arquivo = str_pad($arquivo, 71, '0');

        // CAMPO: Dígito verificador da conta
        // POSIÇÃO: 072 a 72
        // PICTURE: X(1)
        if($contcorr)
            $arquivo .= substr(str_replace(' ', '', $contcorr->getCdcontcorr()), -1);

        // CAMPO: Dígito Verificador da Agência/Conta
        // POSIÇÃO: 073 a 073
        // PICTURE: X(1)
        $arquivo = str_pad($arquivo, 73);

        // CAMPO: Nome da Empresa
        // POSIÇÃO: 074 a 103
        // PICTURE: X(30)
        /** @var $empresa Empresa */
        $empresa = Empresa::findOneByCdempresa($cdempresa);
        $filial = Filial::findOneByCdfilial($cdfilial);
        if($filial) {
            $nmrazsocemp = Util::campoCnabValido($filial->getNmrazsocfili());
            $nmrazsocemp = substr($nmrazsocemp, 0, 30);
            $arquivo .= str_pad($nmrazsocemp, 30);
        } else {
            $nmrazsocemp = Util::campoCnabValido($empresa->getNmrazsocempr());
            $nmrazsocemp = substr($nmrazsocemp, 0, 30);
            $arquivo .= str_pad($nmrazsocemp, 30);
        }
        
        // if ($empresa) {
        //     $nmrazsocemp = Util::campoCnabValido($empresa->getNmrazsocempr());
        //     $nmrazsocemp = substr($nmrazsocemp, 0, 30);
        //     $arquivo .= str_pad($nmrazsocemp, 30);
        // }
        // else
        //     $arquivo = str_pad($arquivo, 102);

        // CAMPO: Mensagem 01
        // POSIÇÃO: 104 a 143
        // PICTURE: X(40)
        $arquivo = str_pad($arquivo, 143);

        // CAMPO: Mensagem 02
        // POSIÇÃO: 144 a 183
        // PICTURE: X(40)
        $arquivo = str_pad($arquivo, 183);

        // CAMPO: Número REMESSA/RETORNO
        // POSIÇÃO: 184 a 191
        // PICTURE: 9(8)
        $arquivo .= substr($nrcnabrec, -8);

        // CAMPO: Data de Gravação REMESSA/RETORNO
        // POSIÇÃO: 192 a 199
        // PICTURE: 9(8)
        $now = DateUtil::getDataDeString(DateUtil::truncateString(DateUtil::getDataAtual()));
        $data = DateUtil::getDia($now) . DateUtil::getMes($now) . DateUtil::getAno($now);
        $arquivo .= Util::campoCnabValido($data);

        // CAMPO: Data do Crédito
        // POSIÇÃO: 200 a 207
        // PICTURE: 9(8)
        $arquivo = str_pad($arquivo, 207, '0');

        // CAMPO: Brancos
        // POSIÇÃO: 208 a 240
        // PICTURE: X(33)
        $arquivo = str_pad($arquivo, 240) . "\r\n";

        //Grava a variavel arquivo no arquivo texto
        fprintf($file, "%s", $arquivo);
        fclose($file);
    }

    //#################################################################//
    //################  Registro Detalhe Segmento P  ##################//
    //#################################################################//
    public function geraDetalheSegmentoP($caminho, $nmarquivo, $nrorg, $cdbanco, $cdempresa, $cdidrec, $nrseqremessarec,
                                                  $nrcnabrec, $idoperacao, $cdagenciacnab, $cdcontacorrente, $cdcontcorrcobranca, $nossonumero,
                                                  $idtipocobranca, $nrlanbolrec, $cdfilial, $dtvenbolrec, $valorfinalboleto, $valorpemobrec, $valormordiabrec,
                                                  $valorpemulbrec, $valormultabrec, $valorfinalbrec, $cdcartcobbol, $cddiaprotbol, $cdreducont, $imprbol,
                                                  $qtdiaprotbol, $contador = NULL
    ){
        if (!is_dir($caminho))
            mkdir($caminho, "0777", true);
        $nmarquivo = $caminho.$nmarquivo;
        $file = fopen($nmarquivo, "a+");

        // CAMPO: Código do Banco
        // POSIÇÃO: 001 a 003
        // PICTURE: 9(3)
        $arquivo = self::CODIGO_BANCO;

        // CAMPO: Lote de Serviço
        // POSIÇÃO: 004 a 007
        // PICTURE: 9(4)
        $arquivo .= self::LOTE_DO_SERVICO;

        // CAMPO: Identificação do registro
        // POSIÇÃO: 008 a 008
        // PICTURE: 9(1)
        $arquivo .= self::TIPO_DE_REGISTRO_DETALHE;

        // CAMPO: Nº seq. do reg. no lote
        // POSIÇÃO: 009 a 013
        // PICTURE: 9(5)
        $contador = str_pad($contador, 5, "0", STR_PAD_LEFT);
        $arquivo .= Util::campoCnabValido($contador);

        // CAMPO: Código do Segmento
        // POSIÇÃO: 014 a 014
        // PICTURE: X(1)
        $arquivo .= self::CODIGO_SEGMENTO_P;

        // CAMPO: Brancos
        // POSIÇÃO: 015 a 015
        // PICTURE: X(1)
        $arquivo = str_pad($arquivo, 15);

        // CAMPO: Código do Movimento
        // POSIÇÃO: 016 a 017
        // PICTURE: 9(2)
        // CONTEÚDO: 01 – Entrada de títulos,                        02 – Pedido de baixa,
        //           04 – Concessão de Abatimento,                   05 – Cancelamento de Abatimento,
        //           06 – Alteração de Vencimento,                   07 – Concessão de Desconto,
        //           08 – Cancelamento de Desconto,                  09 – Protestar,
        //           10 – Cancela/Sustação da Instrução de protesto, 30 – Recusa da Alegação do Sacado,
        //           31 – Alteração de Outros Dados,                 40 – Alteração de Modalidade.
        if ($idoperacao and $idoperacao == 'I')
            $arquivo .= '01'; //Inclusão
        else if ($idoperacao == 'E')
            $arquivo .= '02'; // Exclusão
        else
            $arquivo .= '06'; //Alteração

        // CAMPO: Agência mantenedora da conta
        // POSIÇÃO: 018 a 022
        // PICTURE: X(6)
        /** @var $contcorr Contcorr */
        $contcorr = Contcorr::findOneByCdreducont($cdreducont);
        if($contcorr && $contcorr->getCdagencia())
            $arquivo .= str_pad(substr($contcorr->getCdagencia(), 0, 4), 5, '0', STR_PAD_LEFT);
        else
            $arquivo = str_pad($arquivo, 22, '0');

        // CAMPO: Dígito verificador
        // POSIÇÃO: 023 a 23
        // PICTURE: X(1)
        if($contcorr)
            $arquivo .= substr($contcorr->getCdagencia(), -1);

        // CAMPO: Conta Corrente
        // POSIÇÃO: 024 a 035
        // PICTURE: X(12)
        if($contcorr && $contcorr->getCdcontcorr()){
            $tamanho = strlen(str_replace(' ', '', $contcorr->getCdcontcorr())) - 1;
            $arquivo .= str_pad(substr(str_replace(' ', '', $contcorr->getCdcontcorr()), 0, $tamanho), 12, '0', STR_PAD_LEFT);
        }
        else
            $arquivo = str_pad($arquivo, 35, '0');

        // CAMPO: Dígito verificador da conta
        // POSIÇÃO: 036 a 036
        // PICTURE: X(1)
        if($contcorr)
            $arquivo .= substr(str_replace(' ', '', $contcorr->getCdcontcorr()), -1);

        // CAMPO: Dígito Verificador da Agência/Conta
        // POSIÇÃO: 037 a 037
        // PICTURE: X(1)
        $arquivo = str_pad($arquivo, 37);

        // CAMPO: Nosso Número
        // POSIÇÃO: 038 a 057
        // PICTURE: 9(20)

        $convenio = $contcorr->getCdcedenteban();
        if(strlen($convenio) == 4){
            $arquivo .= str_pad($convenio . substr(str_replace([' ', '-'], '', $nossonumero), -8), 20);
        }
        elseif(strlen($convenio) == 6){
            $arquivo .= str_pad($convenio . substr(str_replace([' ', '-'], '', $nossonumero), -6), 20);
        }
        else{
            $arquivo .= str_pad($convenio . substr(str_replace([' ', '-'], '', $nossonumero), -10), 20);
        }

        // CAMPO: Nº da carteira
        // POSIÇÃO: 058 a 058
        // PICTURE: 9(1)
        // CONTEÚDO:
        //           1      - Carteira 11/12 na Modalidade Simples;
        //           2 ou 3 - Carteira 11/17 na Modalidade Vinculada/Caucionada e Carteira 31;
        //           4      - Carteira 11/17 na Modalidade Descontada e Carteira 51;
        //           7      - Carteira 17 na Modalidade Simples.
        switch($idtipocobranca){
            case '1':
                if(substr($cdcartcobbol, -2) == '17')
                    $arquivo .= '7';
                else
                    $arquivo .= '1';
                break;
            case '2':
                $arquivo .= '2';
                break;
            case '3':
                $arquivo .= '3';
                break;
            case '4':
                $arquivo .= '4';
                break;
        }

        // CAMPO: Forma de Cadastramento
        // POSIÇÃO: 059 a 059
        // PICTURE: X(1)
        /** @var $boleto Boletreceber */
        $boleto = Boletreceber::findOneByNrlanbolrecAndCdfilial($nrlanbolrec, $cdfilial);
        if($boleto and $boleto->getIdformcadbrec())
            $arquivo .= $boleto->getIdformcadbrec();
        else
            $arquivo .= '1';

        // CAMPO: Tipo de Documento
        // POSIÇÃO: 060 a 060
        // PICTURE: X(1)
        $arquivo .= self::TIPO_DOCUMENTO;


        // As carteiras 11 e 51 são tratadas pelo proprio banco
        if ((substr($cdcartcobbol, -2) == '11')  or (substr($cdcartcobbol, -2) == '12') or
            (substr($cdcartcobbol, -2) == '31') or (substr($cdcartcobbol, -2) == '51')) {
                $imprimeBol = '1'; // BANCO EMITE
                $distribBol = '1';
        }
        else {
            if ($imprbol == 'B' or
                !$nossonumero) {
                    $imprimeBol = '1'; // BANCO EMITE
                    $distribBol = '1';
            }
            else {
                $imprimeBol = '2'; // CLIENTE EMITE
                $distribBol = '2';
            }
        }

        // CAMPO: Emissão do Boleto
        // POSIÇÃO: 061 a 061
        // PICTURE: X(1)
        $arquivo .= $imprimeBol;

        // CAMPO: Distribui Boleto
        // POSIÇÃO: 062 a 062
        // PICTURE: X(1)
        $arquivo .= $distribBol;


        // CAMPO: Identificação do título na empresa
        // POSIÇÃO: 063 a 072
        // PICTURE: X(10)
        /** @var $entityBoleto \Financeiro\Repositories\Boletreceber */
        $entityBoleto = Boletreceber::getRepository();
        $duplicata = $entityBoleto->getDuplicata($cdfilial, $nrlanbolrec);

        if (!$duplicata[0]['DUPLICATA'])
            $arquivo .= str_pad($nrlanbolrec, 10, '0');
        else {
            $arquivo .= str_pad(substr($duplicata[0]['DUPLICATA'], -7), 7, '0', STR_PAD_LEFT) . '/' . str_pad($duplicata[0]['NRPARCELA'], 2, '0', STR_PAD_LEFT);
        }

        // POSIÇÃO: 073 a 077
        // PICTURE: X(5)
        $arquivo = str_pad($arquivo, 77);

        // CAMPO: Data de vencimento do título
        // POSIÇÃO: 078 a 085
        // PICTURE: 9(8)
        $vencimento = DateUtil::getDataDeString(DateUtil::truncateString($dtvenbolrec));
        $data = DateUtil::getDia($vencimento) . DateUtil::getMes($vencimento) . DateUtil::getAno($vencimento);
        $arquivo .= Util::campoCnabValido($data);

        // CAMPO: Valor do título
        // POSIÇÃO: 086 a 100
        // PICTURE: 9(13)V9(2)
        $vrfinabolrec = str_pad(($valorfinalboleto * 100), 15, "0", STR_PAD_LEFT);
        $arquivo .= Util::campoCnabValido($vrfinabolrec);

        // CAMPO: Agência cobradora
        // POSIÇÃO: 101 a 105
        // PICTURE: 9(5)
        $arquivo = str_pad($arquivo, 105, '0');

        // CAMPO: Dígito verificador
        // POSIÇÃO: 106 a 106
        // PICTURE: X(1)
        $arquivo = str_pad($arquivo, 106);

        // CAMPO: Espécie do título
        // POSIÇÃO: 107 a 108
        // PICTURE: 9(2)
        $arquivo .= self::ESPECIE_TITULO;

        // CAMPO: Identificação de título aceito ou não aceito
        // POSIÇÃO: 109 a 109
        // PICTURE: X(1)
        $arquivo .= self::ID_TITULO_ACEITO;

        // CAMPO: Data de emissão do título
        // POSIÇÃO: 110 a 117
        // PICTURE: 9(8)
        $emissao = DateUtil::getDataDeString(DateUtil::truncateString($duplicata[0]['DTEMISSAO']));
        $data = DateUtil::getDia($emissao) . DateUtil::getMes($emissao) . DateUtil::getAno($emissao);
        $arquivo .= Util::campoCnabValido($data);

        // CAMPO: Código de Juros de Mora
        // POSIÇÃO: 118 a 118
        // PICTURE: 9(1)
        //1 = Valor por Dia; 2 = Taxa Mensal e 3 = Isento.
        // passar valor 1 cliente expert (valor diário)
        if($valormordiabrec == 0){
            if($valorpemobrec == 0)
                $arquivo .= '3';
            else
                $arquivo .= '2';
        }
        else
            $arquivo .= '1';

        // CAMPO: Data de Juros de Mora
        // POSIÇÃO: 119 a 126
        // PICTURE: 9(8)
        $arquivo = str_pad($arquivo, 126, '0');

        // CAMPO: Valor de mora por dia de atraso
        // POSIÇÃO: 127 a 141
        // PICTURE: 9(13)V9(2)
        if($valormordiabrec == 0){
            if($valorpemobrec == 0)
                $arquivo = str_pad($arquivo, 141, '0');
            else{
                $vrpemorbrec = str_pad(($valorpemobrec * 100), 15, "0", STR_PAD_LEFT);
                $arquivo .= Util::campoCnabValido($vrpemorbrec);
            }
        }
        else{
            $vrmordiabrec = str_pad(($valormordiabrec * 100), 15, "0", STR_PAD_LEFT);
            $arquivo .= Util::campoCnabValido($vrmordiabrec);
        }

        // CAMPO: Código do Desconto
        // POSIÇÃO: 142 a 142
        // PICTURE: 9(1)
        $arquivo .= self::CODIGO_DESCONTO;

        // CAMPO: Data de Desconto
        // POSIÇÃO: 143 a 150
        // PICTURE: 9(8)
        $arquivo = str_pad($arquivo, 150, '0');

        // CAMPO: Valor do Desconto
        // POSIÇÃO: 151 a 165
        // PICTURE: 9(13)V9(2)
        $arquivo = str_pad($arquivo, 165, '0');

        // CAMPO: Valor do I.O.F.
        // POSIÇÃO: 166 a 180
        // PICTURE: 9(13)V9(2)
        $arquivo = str_pad($arquivo, 180, '0');

        // CAMPO: Valor do Abatimento
        // POSIÇÃO: 181 a 195
        // PICTURE: 9(13)V9(2)
        $arquivo = str_pad($arquivo, 195, '0');

        // CAMPO: Identificação do Título na Empresa
        // POSIÇÃO: 196 a 220
        // PICTURE: X(25)
        $arquivo .= str_pad($cdfilial.$nrlanbolrec, 25);

        if(!$cddiaprotbol)
            $diaprotbol = '3';
        else
            $diaprotbol = substr($cddiaprotbol, -1);

        if(!$qtdiaprotbol)
            $quantidadedia = '00';
        else
            $quantidadedia = substr($qtdiaprotbol, -2);

        if ($diaprotbol == '3' or $quantidadedia == '00') {
            $diaprotbol = '3';
            $quantidadedia = '00';
        }

        // CAMPO: Código para Protesto
        // POSIÇÃO: 221 a 221
        // PICTURE: 9(1)
        $arquivo .= $diaprotbol;

        // CAMPO: Prazo para Protesto
        // POSIÇÃO: 222 a 223
        // PICTURE: 9(2)
        $arquivo .= $quantidadedia;

        // CAMPO: Código para Baixa/Devolução
        // POSIÇÃO: 224 a 224
        // PICTURE: 9(1)
        $arquivo .= self::CODIGO_PARA_BAIXA;

        // CAMPO: Prazo para Baixa/Devolução
        // POSIÇÃO: 225 a 227
        // PICTURE: 9(3)
        $arquivo = str_pad($arquivo, 227, '0');

        // CAMPO: Código da moeda
        // POSIÇÃO: 228 a 229
        // PICTURE: 9(2)
        $arquivo .= self::CODIGO_MOEDA;

        // CAMPO: Número do Contrato da Operação
        // POSIÇÃO: 230 a 239
        // PICTURE: X(10)
        $arquivo = str_pad($arquivo, 239, '0');

        // CAMPO: Uso Exclusivo CNAB
        // POSIÇÃO: 240 a 240
        // PICTURE: X(1)
        $arquivo = str_pad($arquivo, 240) . "\r\n";

        //Grava a variavel arquivo no arquivo texto
        fprintf($file, "%s", $arquivo);
        fclose($file);
    }
    public function geraDetalheSegmentoQ($caminho, $nmarquivo, $nrorg, $cdbanco, $cdcliente, $idoperacao, $cdempresa, $cdfilial, $nrlanbolrec, $contador = NULL){
        if (!is_dir($caminho))
            mkdir($caminho, "0777", true);
        $nmarquivo = $caminho.$nmarquivo;
        $file = fopen($nmarquivo, "a+");

        // CAMPO: Código do Banco
        // POSIÇÃO: 001 a 003
        // PICTURE: 9(3)
        $arquivo = self::CODIGO_BANCO;

        // CAMPO: Lote de Serviço
        // POSIÇÃO: 004 a 007
        // PICTURE: 9(4)
        $arquivo .= self::LOTE_DO_SERVICO;

        // CAMPO: Identificação do registro
        // POSIÇÃO: 008 a 008
        // PICTURE: 9(1)
        $arquivo .= self::TIPO_DE_REGISTRO_DETALHE;

        // CAMPO: Nº seq. do reg. no lote
        // POSIÇÃO: 009 a 013
        // PICTURE: 9(5)
        $contador = str_pad($contador, 5, "0", STR_PAD_LEFT);
        $arquivo .= Util::campoCnabValido($contador);

        // CAMPO: Código do Segmento
        // POSIÇÃO: 014 a 014
        // PICTURE: X(1)
        $arquivo .= self::CODIGO_SEGMENTO_Q;

        // CAMPO: Brancos
        // POSIÇÃO: 015 a 015
        // PICTURE: X(1)
        $arquivo = str_pad($arquivo, 15);

        // CAMPO: Código do Movimento
        // POSIÇÃO: 016 a 017
        // PICTURE: 9(2)
        // CONTEÚDO: 01 – Entrada de títulos,                        02 – Pedido de baixa,
        //           04 – Concessão de Abatimento,                   05 – Cancelamento de Abatimento,
        //           06 – Alteração de Vencimento,                   07 – Concessão de Desconto,
        //           08 – Cancelamento de Desconto,                  09 – Protestar,
        //           10 – Cancela/Sustação da Instrução de protesto, 30 – Recusa da Alegação do Sacado,
        //           31 – Alteração de Outros Dados,                 40 – Alteração de Modalidade.
        if ($idoperacao and $idoperacao == 'I')
            $arquivo .= '01'; //Inclusão
        else if ($idoperacao == 'E')
            $arquivo .= '02'; // Exclusão
        else
            $arquivo .= '06'; //Alteração

        /** @var $entityLicencaUso \Geral\Repositories\Licencauso */
        $entityLicencaUso = Licencauso::getRepository();

        $licencauso = $entityLicencaUso->getLicencaUso($cdfilial, $nrlanbolrec);
        if($licencauso){
            if($licencauso[0]['IDTPIJURLUSO'] == 'F')
                $tipoSacado = '1'; //CPF
            else
                $tipoSacado = '2'; //CNPJ

            if($licencauso[0]['NRINSJURLUSO'])
                $nrinsjur = $licencauso[0]['NRINSJURLUSO'];
            if(isset($licencauso[0]['NRCEPLUSO']))
                $nrcep = $licencauso[0]['NRCEPLUSO'];
            else
                $nrcep = '00000000';

            $nmclie = $licencauso[0]['NMFANTLUSO'];
            $dsendeclie = $licencauso[0]['DSENDELUSO'];
            $nmbairro = $licencauso[0]['NMBAIRLUSO'];
            $nmmunicipio = $licencauso[0]['NMMUNICIPIO'];
            $sgestado = $licencauso[0]['SGESTADO'];
        }
        else{
            /** @var $entityEndeClie \Geral\Repositories\Endeclie */
            // $entityEndeClie = Endeclie::getRepository();
            $endeclie = Endeclie::findOneByNrorgAndCdclienteAndIdtpendeclie($nrorg, $cdcliente, 'C');
            
            /** @var $boleto Boletreceber */
            $boleto = Boletreceber::findOneByNrlanbolrecAndCdfilial($nrlanbolrec, $cdfilial);
            /** @var $cliente Cliente */
            $cliente = Cliente::findOneByCdclienteAndNrorg($boleto->getCdcliente(), $nrorg);

            // $endeclie = $entityEndeClie->getEndeClie($cdcliente);

            if($cliente->getIdtpijurclie() == 'F')
                $tipoSacado = '1'; //CPF
            else
                $tipoSacado = '2'; //CNPJ

            if($cliente->getNrinsjurclie())
                $nrinsjur = $cliente->getNrinsjurclie();
                
            if($endeclie->getNrcepclie()){
                $nrcep = $endeclie->getNrcepclie();
            } else {
                $nrcep = '00000000';
            }
            
            // if(isset($endeclie[0]['NRCEPCLIE']))
            //     $nrcep = $endeclie[0]['NRCEPCLIE'];
            // else
            //     $nrcep = '00000000';

            $nmclie = $cliente->getNmrazsocclie();
            $dsendeclie = $endeclie->getDsendeclie();
            $nmbairro = $endeclie->getNmbairclie();
            $nmmunicipio = $endeclie->getMunicipio();
            $sgestado = $endeclie->getSgestado();
           
            
            // $dsendeclie = $endeclie[0]['DSENDECLIE'];
            // $nmbairro = $endeclie[0]['NMBAIRCLIE'];
            // $nmmunicipio = $endeclie[0]['NMMUNICIPIO'];
            // $sgestado = $endeclie[0]['SGESTADO'];

        }

        // CAMPO: Código de inscrição/Sacado ('1' = CPF, '2' = CNPJ)
        // POSIÇÃO: 018 a 018
        // PICTURE: 9(1)
        $arquivo .= $tipoSacado;

        // CAMPO: Nº de inscrição do sacado (CPF/CGC)
        // POSIÇÃO: 019 a 033
        // PICTURE: 9(15)
        $arquivo .= Util::campoCnabValido(str_pad($nrinsjur, 15, '0', STR_PAD_LEFT));

        // CAMPO: Nome do sacado
        // POSIÇÃO: 034 a 073
        // PICTURE: X(40)
        $nomecliente = substr(Util::campoCnabValido($nmclie), 0, 40);
        $arquivo .= str_pad($nomecliente, 40);

        // CAMPO: Logradouro (Endereço do sacado (cliente))
        // POSIÇÃO: 074 a 113
        // PICTURE: X(40)
        $enderecoCliente = substr(Util::campoCnabValido($dsendeclie), 0, 40);
        $arquivo .= str_pad($enderecoCliente, 40);


        // CAMPO: Bairro do sacado
        // POSIÇÃO: 114 a 128
        // PICTURE: X(15)
        $bairro = substr(Util::campoCnabValido($nmbairro), 0, 15);
        $arquivo .= str_pad($bairro, 15);

        // CAMPO: C.E.P. do sacado
        // POSIÇÃO: 129 a 136
        // PICTURE: 9(8)
        $arquivo .= Util::campoCepTel($nrcep);

        // CAMPO: Cidade do sacado
        // POSIÇÃO: 137 a 151
        // PICTURE: X(15)
        
        $municipio = substr(Util::campoCnabValido($nmmunicipio->getNmmunicipio()), 0, 15);
        $arquivo .= str_pad($municipio, 15);

        // CAMPO: Estado do sacado
        // POSIÇÃO: 152 a 153
        // PICTURE: X(2)
        $arquivo .= Util::campoCnabValido(Util::campoCnabValido($sgestado));

        // CAMPO: Código de inscrição do sacador ou avalista
        // POSIÇÃO: 154 a 154
        // PICTURE: 9(1)
        $arquivo = str_pad($arquivo, 154, '0');

        // CAMPO: Nº de inscrição do sacador ou avalista
        // POSIÇÃO: 155 a 169
        // PICTURE: 9(15)
        $arquivo = str_pad($arquivo, 169, '0');

        // CAMPO: Nome do sacador ou avalista
        // POSIÇÃO: 170 a 209
        // PICTURE: X(40)
        $arquivo = str_pad($arquivo, 209);

        // CAMPO: Código do Banco correspondente
        // POSIÇÃO: 210 a 212
        // PICTURE: 9(3)
        $arquivo = str_pad($arquivo, 212, '0');

        // CAMPO: Nosso Número do Banco correspondente
        // POSIÇÃO: 213 a 232
        // PICTURE: X(20)
        $arquivo = str_pad($arquivo, 232);

        // CAMPO: Uso exclusivo FEBRABAN
        // POSIÇÃO: 233 a 240
        // PICTURE: X(8)
        $arquivo = str_pad($arquivo, 240) . "\r\n";

        //Grava a variavel arquivo no arquivo texto
        fprintf($file, "%s", $arquivo);
        fclose($file);
    }
    public function geraDetalheSegmentoR($caminho, $nmarquivo, $nrorg, $cdbanco, $idoperacao, $dtvenbolrec, $valorpemulbrec, $nrlanbolrec, $valormultabrec, $contador = NULL){

        if (!is_dir($caminho))
            mkdir($caminho, "0777", true);
        $nmarquivo = $caminho.$nmarquivo;
        $file = fopen($nmarquivo, "a+");

        // CAMPO: Código do Banco
        // POSIÇÃO: 001 a 003
        // PICTURE: 9(3)
        $arquivo = self::CODIGO_BANCO;

        // CAMPO: Lote de Serviço
        // POSIÇÃO: 004 a 007
        // PICTURE: 9(4)
        $arquivo .= self::LOTE_DO_SERVICO;

        // CAMPO: Identificação do registro
        // POSIÇÃO: 008 a 008
        // PICTURE: 9(1)
        $arquivo .= self::TIPO_DE_REGISTRO_DETALHE;

        // CAMPO: Nº seq. do reg. no lote
        // POSIÇÃO: 009 a 013
        // PICTURE: 9(5)
        $contador = str_pad($contador, 5, "0", STR_PAD_LEFT);
        $arquivo .= Util::campoCnabValido($contador);

        // CAMPO: Código do Segmento
        // POSIÇÃO: 014 a 014
        // PICTURE: 9(1)
        $arquivo .= self::CODIGO_SEGMENTO_R;

        // CAMPO: Brancos
        // POSIÇÃO: 015 a 015
        // PICTURE: X(1)
        $arquivo = str_pad($arquivo, 15);

        // CAMPO: Código de Movimento
        // POSIÇÃO: 016 a 017
        // PICTURE: 9(2)
        // CONTEÚDO: 01 – Entrada de títulos,                        02 – Pedido de baixa,
        //           04 – Concessão de Abatimento,                   05 – Cancelamento de Abatimento,
        //           06 – Alteração de Vencimento,                   07 – Concessão de Desconto,
        //           08 – Cancelamento de Desconto,                  09 – Protestar,
        //           10 – Cancela/Sustação da Instrução de protesto, 30 – Recusa da Alegação do Sacado,
        //           31 – Alteração de Outros Dados,                 40 – Alteração de Modalidade.
        if ($idoperacao and $idoperacao == 'I')
            $arquivo .= '01'; //Inclusão
        else if ($idoperacao == 'E')
            $arquivo .= '02'; // Exclusão
        else
            $arquivo .= '06'; //Alteração

        // CAMPO: Código de desconto 2
        // POSIÇÃO: 018 a 018
        // PICTURE: 9(1)
        $arquivo .= self::CODIGO_DESCONTO;

        // CAMPO: Data do desconto 2
        // POSIÇÃO: 019 a 026
        // PICTURE: 9(8)
        $arquivo = str_pad($arquivo, 26, '0');

        // CAMPO: Valor / percentual
        // POSIÇÃO: 027 a 041
        // PICTURE: 9(15)
        $arquivo = str_pad($arquivo, 41, '0');

        // CAMPO: Código de desconto 3
        // POSIÇÃO: 042 a 042
        // PICTURE: 9(1)
        $arquivo .= self::CODIGO_DESCONTO;

        // CAMPO: Data do desconto 3
        // POSIÇÃO: 043 a 050
        // PICTURE: 9(8)
        $arquivo = str_pad($arquivo, 50, '0');

        // CAMPO: Valor / percentual concedido
        // POSIÇÃO: 051 a 065
        // PICTURE: 9(15)
        $arquivo = str_pad($arquivo, 65, '0');

        // CAMPO: Código da multa
        // POSIÇÃO: 066 a 066
        // PICTURE: 9(1)
        if($valorpemulbrec != '0')
            $arquivo .= '2';  //percental
        else if ($valormultabrec != '0')
            $arquivo .= '1';  //fixo
        else
            $arquivo .= '0'; //nao tem

        // CAMPO: Data da multa
        // POSIÇÃO: 067 a 074
        // PICTURE: 9(8)
        if($valormultabrec == 0 && $valorpemulbrec == 0)
            $arquivo = str_pad($arquivo, 74, '0');
        else{
            $vencimento = DateUtil::getDataDeString(DateUtil::truncateString($dtvenbolrec));
            $vencimento = DateUtil::adicionaIntervalo($vencimento, 1);
            $data = DateUtil::getDia($vencimento) . DateUtil::getMes($vencimento) . DateUtil::getAno($vencimento);
            $arquivo .= Util::campoCnabValido($data);
        }

        // CAMPO: Valor / percentual
        // POSIÇÃO: 075 a 089
        // PICTURE: 9(15)
        if ($valorpemulbrec == 0){
            if ($valormultabrec == 0){
				$arquivo = str_pad($arquivo, 89, '0');
            }else{
				$vrmordiabrec = str_pad(($valormultabrec * 100), 15, "0", STR_PAD_LEFT);
				$arquivo .= Util::campoCnabValido($vrmordiabrec);
            }
        }else{
		    $vrpemorbrec = str_pad(($valorpemulbrec * 100), 15, "0", STR_PAD_LEFT);
            $arquivo .= Util::campoCnabValido($vrpemorbrec);
		}

        // CAMPO: Informação do banco ao sacado
        // POSIÇÃO: 090 a 099
        // PICTURE: 9(10)
        $arquivo = str_pad($arquivo, 99);

        // CAMPO: Mensagem 3
        // POSIÇÃO: 100 a 139
        // PICTURE: 9(40)
        $arquivo = str_pad($arquivo, 139);

        // CAMPO: Mensagem 4
        // POSIÇÃO: 140 a 179
        // PICTURE: 9(40)
        $arquivo = str_pad($arquivo, 179);

        // CAMPO: Código do banco da conta
        // POSIÇÃO: 180 a 182
        // PICTURE: 9(3)
        $arquivo = str_pad($arquivo, 182, '0');

        // CAMPO: Código da agencia
        // POSIÇÃO: 183 a 186
        // PICTURE: 9(4)
        $arquivo = str_pad($arquivo, 186, '0');

        // CAMPO: Conta Corrente
        // POSIÇÃO: 187 a 199
        // PICTURE: 9(13)
        $arquivo = str_pad($arquivo, 199, '0');

        // CAMPO: Códigos de ocorrencia
        // POSIÇÃO: 200 a 207
        // PICTURE: 9(8)
        $arquivo = str_pad($arquivo, 207, '0');

        // CAMPO: Uso exclusivo febraban/cnab
        // POSIÇÃO: 208 a 240
        // PICTURE: 9(33)
        $arquivo = str_pad($arquivo, 240) . "\r\n";

        //Grava a variavel arquivo no arquivo texto
        fprintf($file, "%s", $arquivo);
        fclose($file);
    }
    public function GeraTraillerLote($caminho, $nmarquivo, $nrorg, $cdbanco, $contreg = NULL){
        if (!is_dir($caminho))
            mkdir($caminho, "0777", true);
        $nmarquivo = $caminho.$nmarquivo;
        $file = fopen($nmarquivo, "a+");

        // CAMPO: Código do Banco
        // POSIÇÃO: 001 a 003
        // PICTURE: 9(3)
        $arquivo = self::CODIGO_BANCO;

        // CAMPO: Lote de Serviço
        // POSIÇÃO: 004 a 007
        // PICTURE: 9(4)
        $arquivo .= self::LOTE_DO_SERVICO;

        // CAMPO: Identificação do registro
        // POSIÇÃO: 008 a 008
        // PICTURE: 9(1)
        $arquivo .= self::TIPO_DE_REGISTRO_TRAILLER_LOTE;

        // CAMPO: Uso exclusivo FEBRABAN
        // POSIÇÃO: 009 a 017
        // PICTURE: X(9)
        $arquivo = str_pad($arquivo, 17);

        // CAMPO: Quantidade de registros do arquivo
        // POSIÇÃO: 018 a 023
        // PICTURE: 9(6)
        $arquivo .= Util::campoCnabValido(str_pad($contreg - 1, 6, "0", STR_PAD_LEFT));

        // CAMPO: Quantidade de títulos em cobrança
        // POSIÇÃO: 024 a 029
        // PICTURE: 9(6)
        $arquivo = str_pad($arquivo, 29, '0');

        // CAMPO: Valor total de títulos em carteiras
        // POSIÇÃO: 030 a 046
        // PICTURE: 9(15)V9(2)
        $arquivo = str_pad($arquivo, 46, '0');

        // CAMPO: Quantidade de títulos em cobrança
        // POSIÇÃO: 047 a 052
        // PICTURE: 9(6)
        $arquivo = str_pad($arquivo, 52, '0');

        // CAMPO: Valor total de títulos em carteiras
        // POSIÇÃO: 053 a 069
        // PICTURE: 9(15)V9(2)
        $arquivo = str_pad($arquivo, 69, '0');

        // CAMPO: Quantidade de títulos em cobrança
        // POSIÇÃO: 070 a 075
        // PICTURE: 9(6)
        $arquivo = str_pad($arquivo, 75, '0');

        // CAMPO: Valor total de títulos em carteiras
        // POSIÇÃO: 076 a 092
        // PICTURE: 9(15)V9(2)
        $arquivo = str_pad($arquivo, 92, '0');

        // CAMPO: Quantidade de títulos em cobrança
        // POSIÇÃO: 093 a 098
        // PICTURE: 9(6)
        $arquivo = str_pad($arquivo, 98, '0');

        // CAMPO: Valor total de títulos em carteiras
        // POSIÇÃO: 099 a 115
        // PICTURE: 9(15)V9(2)
        $arquivo = str_pad($arquivo, 115, '0');

        // CAMPO: Número do aviso de lançamento
        // POSIÇÃO: 116 a 123
        // PICTURE: X(8)
        $arquivo = str_pad($arquivo, 123);

        // CAMPO: Brancos
        // POSIÇÃO: 124 a 240
        // PICTURE: X(117)
        $arquivo = str_pad($arquivo, 240) . "\r\n";

        //Grava a variavel arquivo no arquivo texto
        fprintf($file, "%s", $arquivo);
        fclose($file);
    }
    public function GeraTrailler($caminho, $nmarquivo, $nrorg, $cdbanco, $contlote = NULL, $contreg = NULL, $cdreducont = NULL){
        if (!is_dir($caminho))
            mkdir($caminho, "0777", true);
        $nmarquivo = $caminho.$nmarquivo;
        $file = fopen($nmarquivo, "a+");

        // CAMPO: Código do Banco
        // POSIÇÃO: 001 a 003
        // PICTURE: 9(3)
        $arquivo = self::CODIGO_BANCO;

        // CAMPO: Lote de Serviço
        // POSIÇÃO: 004 a 007
        // PICTURE: 9(4)
        $arquivo .= self::LOTE_DO_SERVICO_TRAILLER;

        // CAMPO: Identificação do Arquivo
        // POSIÇÃO: 008 a 008
        // PICTURE: 9(1)
        $arquivo .= self::TIPO_DE_REGISTRO_TRAILLER;

        // CAMPO: Uso exclusivo FEBRABAN
        // POSIÇÃO: 009 a 017
        // PICTURE: X(9)
        $arquivo = str_pad($arquivo, 17);

        // CAMPO: Quantidade de lotes do arquivo
        // POSIÇÃO: 018 a 023
        // PICTURE: 9(6)
        $arquivo .= Util::campoCnabValido(str_pad($contlote, 6, "0", STR_PAD_LEFT));

        // CAMPO: Quantidade de registros do arquivo
        // POSIÇÃO: 024 a 029
        // PICTURE: 9(6)
        $arquivo .= Util::campoCnabValido(str_pad($contreg - 1, 6, "0", STR_PAD_LEFT));

        // CAMPO: Quantidade de contas p/ conciliação
        // POSIÇÃO: 030 a 035
        // PICTURE: 9(6)
        $arquivo = str_pad($arquivo, 35, '0');

        // CAMPO: Brancos
        // POSIÇÃO: 036 a 240
        // PICTURE: X(205)
        $arquivo = str_pad($arquivo, 240) . "\r\n";

        //Grava a variavel arquivo no arquivo texto
        fprintf($file, "%s", $arquivo);
        fclose($file);

    }
    public function geraTransacao($caminho, $nmarquivo,$nrorg, $cdempresa, $cdcartcobbol, $cdagencia, $cdcontcorr, $cdfilial,
                                           $nrlanbolrec, $vrpemobrec, $idimprboleto, $cdnosnumrec, $dtvenbolrec, $cdins1cobbol,
                                           $qtdiaprotbol, $vrmordiabrec, $vrfinalbrec, $idopecnabrec, $vrpemulbrec, $idtpijurclie,
                                           $nrinsjurclie, $cdcliente, $cdins2cobbol, $contreg = NULL, $idtipcnabrec = null, $idfilialcnabrec, $cddiaprotbol = NULL, $contador = NULL){}

}