<?php

namespace Teknisa\FIN\Modules\MovimentacaoBancaria\Factories;

use Doctrine\ORM\EntityManager;
use Teknisa\Libs\Util\DateUtil;
use Teknisa\Model\Financeiro\Entities\Lancdespcaixa;
use Teknisa\Model\Contabilidade\Entities\Lanccont;
use Zeedhi\Framework\DB\StoredProcedure\Param;
use Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;

/**
 * Class Bco56200LancamentoDespesaFactory
 * @package Teknisa\FIN\Modules\MovimentacaoBancaria\Factories
 */
class Bco56200LancamentoDespesaFactory { 
    
    protected $entityManager;

    public function __construct(EntityManager $entityManager) {
        $this->connection = $entityManager->getConnection();
        $this->entityManager = $entityManager;
    }
    
    /**
     * Busca Lançamento de Despesa
     * @param $nrorg
     * @param $nrseqlancdesp
     * @return Lancdespcaixa
     */
    public function buscaLancdespcaixa($nrorg, $nrseqlancdesp){
        /** @var $params */
        $params = array();
        $params['nrorg']          = $nrorg;
        $params['nrseqlancdesp']  = $nrseqlancdesp;

        /** @var Lancdespcaixa $lancdespcaixa */
        $lancdespcaixa = $this->entityManager->getRepository(Lancdespcaixa::class)->findOneBy($params);

        return $lancdespcaixa;
    }
    
    /**
     * Busca Lançamento Contabil
     * @param $nrorg
     * @param $cdempresa
     * @param $nrlanccont
     * @return Lanccont
     */
    public function buscaLanccont($nrorg, $cdempresa, $nrlanccont){
        /** @var $params */
        $params = array();
        $params['nrorg']      = $nrorg;
        $params['cdempresa']  = $cdempresa;
        $params['nrlanccont'] = $nrlanccont;

        /** @var Lanccont $lanccont */
        $lanccont = $this->entityManager->getRepository(Lanccont::class)->findBy($params);

        return $lanccont;
    }
    
    /**
     * Insere Lançamento de Despesa Caixa
     * @param $nrorg
     * @param $nrseqlancdesp
     * @param $cdtipodesp
     * @param $dslancdesp
     * @param $cdempresa
     * @param $dtlancdesp
     * @param $cdfilial
     * @param $cdcentcust
     * @param $vrlancdesp
     * @return Lancdespcaixa
     */
    public function insereLancdespcaixa(
        $nrorg, $nrseqlancdesp, $cdtipodesp, $dslancdesp, $cdempresa,
        $dtlancdesp, $cdfilial, $cdcentcust, $vrlancdesp
    ){
        /** @var Lancdespcaixa $lancdespcaixa */
        $lancdespcaixa = new Lancdespcaixa();
        $lancdespcaixa->setNrorg($nrorg);
        $lancdespcaixa->setNrseqlancdesp($nrseqlancdesp);
        $lancdespcaixa->setCdtipodesp($cdtipodesp);
        $lancdespcaixa->setDslancdesp($dslancdesp);
        $lancdespcaixa->setCdempresa($cdempresa);
        $lancdespcaixa->setDtlancdesp($dtlancdesp);
        $lancdespcaixa->setCdfilial($cdfilial);
        $lancdespcaixa->setCdcentcust($cdcentcust);
        $lancdespcaixa->setVrlancdesp($vrlancdesp);
        $lancdespcaixa->setDtinclusao(DateUtil::getDataAtual());

        return $lancdespcaixa;
    }
    
    /**
     * Executa Procedure ATUALIZA_SALDO_CAIXA
     * @param $cdfilial
     * @param $cdcentcust
     * @param $nrseqlancdesp
     * @return mixed
     */
    public function execProcAtualizaSaldoCaixa(
        $cdfilial, $cdcentcust, $nrseqlancdesp, $idatualiza
    ){
        $procedure = new StoredProcedure($this->entityManager->getConnection(), 'ATUALIZA_SALDO_CAIXA');
        $procedure->addParam(new Param('P_CDFILIAL'     , Param::PARAM_INPUT , $cdfilial     , Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDCENTCUST'   , Param::PARAM_INPUT , $cdcentcust   , Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_NRSEQLANCDESP', Param::PARAM_INPUT , $nrseqlancdesp, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_IDATUALIZA'   , Param::PARAM_INPUT , $idatualiza   , Param::PARAM_TYPE_STR));
        $procedure->execute();
    }
    
    /**
     * Busca Saldo do Caixa Disponível
     * @param $nrorg
     * @param $cdfilial
     * @param $nrbaixarec
     * @return array
     */
    public function buscaSaldoCaixaDisp(
        $nrorg, $cdfilial, $cdcentcust, $dtlancdesp
    ){
        $params = array();

        $params['P_NRORG'  ]    = $nrorg;
        $params['P_CDFILIAL'  ] = $cdfilial;
        $params['P_CDCENTCUST'] = $cdcentcust;
        $params['P_DTLANCSALD'] = $dtlancdesp;

        $sql = "SELECT SALDODISP, TO_CHAR(SALDODISP, 'FM999G999G990D90', 'NLS_NUMERIC_CHARACTERS='',.''') AS FORMATSALDODISP
                  FROM (
                    SELECT SUM(VRLANCSALDDISP) AS SALDODISP
                      FROM LANCSALDCAIXA
                     WHERE NRORG       = :P_NRORG
                       AND CDFILIAL    = :P_CDFILIAL
                       AND CDCENTCUST  = :P_CDCENTCUST
                       AND DTLANCSALD <= :P_DTLANCSALD
                )";

        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetch();
    }
}